@@ -45,7 +45,7 @@
 #include <QtPositioning/QGeoCircle>
 #include <QtPositioning/QGeoRectangle>
 #include <QtQuick/QQuickWindow>
-#include <QtQuick/QSGSimpleRectNode>
+#include <QtQuick/QSGRectangleNode>
 #include <QtQuick/private/qquickwindow_p.h>
 #include <QtQml/qqmlinfo.h>
 #include <cmath>
@@ -306,10 +306,10 @@ void QDeclarativeGeoMap::initialize()
     // try to keep center change signal in the end
     bool centerHasChanged = false;
 
-    setMinimumZoomLevel(m_map->minimumZoomForMapSize(width(), height()));
+    setMinimumZoomLevel(m_map->minimumZoomAtViewportSize(width(), height()));
 
     // set latitude bundary check
-    m_maximumViewportLatitude = m_map->maximumLatitudeForZoom(m_cameraData.zoomLevel());
+    m_maximumViewportLatitude = m_map->maximumCenterLatitudeAtZoom(m_cameraData.zoomLevel());
     QGeoCoordinate center = m_cameraData.center();
     center.setLatitude(qBound(-m_maximumViewportLatitude, center.latitude(), m_maximumViewportLatitude));
 
@@ -393,7 +393,6 @@ void QDeclarativeGeoMap::mouseReleaseEvent(QMouseEvent *event)
 {
     if (isInteractive()) {
         m_gestureArea->handleMouseReleaseEvent(event);
-        ungrabMouse();
     } else {
         QQuickItem::mouseReleaseEvent(event);
     }
@@ -476,13 +475,12 @@ QSGNode *QDeclarativeGeoMap::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDa
         return 0;
     }
 
-    QSGSimpleRectNode *root = static_cast<QSGSimpleRectNode *>(oldNode);
+    QSGRectangleNode *root = static_cast<QSGRectangleNode *>(oldNode);
     if (!root)
-        root = new QSGSimpleRectNode(boundingRect(), m_color);
-    else {
-        root->setRect(boundingRect());
-        root->setColor(m_color);
-    }
+        root = window()->createRectangleNode();
+
+    root->setRect(boundingRect());
+    root->setColor(m_color);
 
     QSGNode *content = root->childCount() ? root->firstChild() : 0;
     content = m_map->updateSceneGraph(content, window());
@@ -561,7 +559,7 @@ void QDeclarativeGeoMap::mappingManagerInitialized()
     // after this has been called at least once, after creation.
 
     if (!m_initialized && width() > 0 && height() > 0) {
-        m_map->setSize(QSize(width(), height()));
+        m_map->setViewportSize(QSize(width(), height()));
         initialize();
     }
 
@@ -617,7 +615,7 @@ void QDeclarativeGeoMap::setMinimumZoomLevel(qreal minimumZoomLevel)
 
         if (m_map) {
             minimumZoomLevel = qBound(qreal(m_map->cameraCapabilities().minimumZoomLevel()), minimumZoomLevel, maximumZoomLevel());
-            double minimumViewportZoomLevel = m_map->minimumZoomForMapSize(width(),height());
+            double minimumViewportZoomLevel = m_map->minimumZoomAtViewportSize(width(),height());
             if (minimumZoomLevel < minimumViewportZoomLevel)
                 minimumZoomLevel = minimumViewportZoomLevel;
         }
@@ -716,7 +714,7 @@ void QDeclarativeGeoMap::setZoomLevel(qreal zoomLevel)
 
     if (m_initialized) {
         m_cameraData.setZoomLevel(qBound(minimumZoomLevel(), zoomLevel, maximumZoomLevel()));
-        m_maximumViewportLatitude = m_map->maximumLatitudeForZoom(m_cameraData.zoomLevel());
+        m_maximumViewportLatitude = m_map->maximumCenterLatitudeAtZoom(m_cameraData.zoomLevel());
         QGeoCoordinate coord = m_cameraData.center();
         coord.setLatitude(qBound(-m_maximumViewportLatitude, coord.latitude(), m_maximumViewportLatitude));
         if (coord != m_cameraData.center()) {
@@ -890,6 +888,9 @@ void QDeclarativeGeoMap::fitViewportToGeoShape()
         QGeoRectangle rect = m_region;
         topLeft = rect.topLeft();
         bottomRight = rect.bottomRight();
+        if (bottomRight.longitude() < topLeft.longitude())
+            bottomRight.setLongitude(bottomRight.longitude() + 360.0);
+
         break;
     }
     case QGeoShape::CircleType:
@@ -969,8 +970,7 @@ void QDeclarativeGeoMap::fitViewportToGeoShape()
                             bboxHeight / (height() - margins));
     // fixme: use log2 with c++11
     zoomRatio = std::log(zoomRatio) / std::log(2.0);
-    double newZoom = qMax(minimumZoomLevel(), zoomLevel()
-                          - zoomRatio);
+    double newZoom = qMax<double>(minimumZoomLevel(), zoomLevel() - zoomRatio);
     setZoomLevel(newZoom);
     m_validRegion = true;
 }
@@ -1038,8 +1038,8 @@ void QDeclarativeGeoMap::pan(int dx, int dy)
     if (dx == 0 && dy == 0)
         return;
     QGeoCoordinate coord = m_map->itemPositionToCoordinate(
-                                QDoubleVector2D(m_map->width() / 2 + dx,
-                                        m_map->height() / 2 + dy));
+                                QDoubleVector2D(m_map->viewportWidth() / 2 + dx,
+                                        m_map->viewportHeight() / 2 + dy));
     setCenter(coord);
 }
 
@@ -1113,10 +1113,6 @@ void QDeclarativeGeoMap::touchEvent(QTouchEvent *event)
 {
     if (isInteractive()) {
         m_gestureArea->handleTouchEvent(event);
-        if ( event->type() == QEvent::TouchEnd ||
-             event->type() == QEvent::TouchCancel) {
-            ungrabTouchPoints();
-        }
     } else {
         //ignore event so sythesized event is generated;
         QQuickItem::touchEvent(event);
@@ -1206,13 +1202,11 @@ void QDeclarativeGeoMap::addMapItem(QDeclarativeGeoMapItemBase *item)
 {
     if (!item || item->quickMap())
         return;
-    m_updateMutex.lock();
     item->setParentItem(this);
     if (m_map)
         item->setMap(this, m_map);
     m_mapItems.append(item);
     emit mapItemsChanged();
-    m_updateMutex.unlock();
 }
 
 /*!
@@ -1252,13 +1246,11 @@ void QDeclarativeGeoMap::removeMapItem(QDeclarativeGeoMapItemBase *ptr)
     QPointer<QDeclarativeGeoMapItemBase> item(ptr);
     if (!m_mapItems.contains(item))
         return;
-    m_updateMutex.lock();
     item.data()->setParentItem(0);
     item.data()->setMap(0, 0);
     // these can be optimized for perf, as we already check the 'contains' above
     m_mapItems.removeOne(item);
     emit mapItemsChanged();
-    m_updateMutex.unlock();
 }
 
 /*!
@@ -1272,7 +1264,6 @@ void QDeclarativeGeoMap::clearMapItems()
 {
     if (m_mapItems.isEmpty())
         return;
-    m_updateMutex.lock();
     for (int i = 0; i < m_mapItems.count(); ++i) {
         if (m_mapItems.at(i)) {
             m_mapItems.at(i).data()->setParentItem(0);
@@ -1281,7 +1272,6 @@ void QDeclarativeGeoMap::clearMapItems()
     }
     m_mapItems.clear();
     emit mapItemsChanged();
-    m_updateMutex.unlock();
 }
 
 /*!
@@ -1320,12 +1310,12 @@ void QDeclarativeGeoMap::geometryChanged(const QRectF &newGeometry, const QRectF
     if (!m_map || !newGeometry.size().isValid())
         return;
 
-    m_map->setSize(newGeometry.size().toSize());
+    m_map->setViewportSize(newGeometry.size().toSize());
 
     if (!m_initialized)
         initialize();
     else
-        setMinimumZoomLevel(m_map->minimumZoomForMapSize(newGeometry.width(), newGeometry.height()));
+        setMinimumZoomLevel(m_map->minimumZoomAtViewportSize(newGeometry.width(), newGeometry.height()));
 
     /*!
         The fitViewportTo*() functions depend on a valid map geometry.
@@ -1493,19 +1483,21 @@ bool QDeclarativeGeoMap::sendMouseEvent(QMouseEvent *event)
         }
     }
 
-    if (event->type() == QEvent::MouseButtonRelease) {
-        if (win && win->mouseGrabberItem() == this)
-            ungrabMouse();
-    }
-
     return false;
 }
 
 bool QDeclarativeGeoMap::sendTouchEvent(QTouchEvent *event)
 {
-    QQuickWindowPrivate *win = window() ? QQuickWindowPrivate::get(window()) : 0;
+    const QQuickPointerDevice *touchDevice = QQuickPointerDevice::touchDevice(event->device());
     const QTouchEvent::TouchPoint &point = event->touchPoints().first();
-    QQuickItem *grabber = win ? win->itemForTouchPointId.value(point.id()) : 0;
+
+    auto touchPointGrabberItem = [touchDevice](const QTouchEvent::TouchPoint &point) -> QQuickItem* {
+        if (QQuickEventPoint *eventPointer = touchDevice->pointerEvent()->pointById(point.id()))
+            return eventPointer->grabber();
+        return nullptr;
+    };
+
+    QQuickItem *grabber = touchPointGrabberItem(point);
 
     bool stealEvent = m_gestureArea->isActive();
     bool containsPoint = contains(mapFromScene(point.scenePos()));
@@ -1517,7 +1509,7 @@ bool QDeclarativeGeoMap::sendTouchEvent(QTouchEvent *event)
 
         m_gestureArea->handleTouchEvent(touchEvent.data());
         stealEvent = m_gestureArea->isActive();
-        grabber = win ? win->itemForTouchPointId.value(point.id()) : 0;
+        grabber = touchPointGrabberItem(point);
 
         if (grabber && stealEvent && !grabber->keepTouchGrab() && grabber != this) {
             QVector<int> ids;
@@ -1538,11 +1530,6 @@ bool QDeclarativeGeoMap::sendTouchEvent(QTouchEvent *event)
         }
     }
 
-    if (event->type() == QEvent::TouchEnd) {
-        if (win && win->itemForTouchPointId.value(point.id()) == this) {
-            ungrabTouchPoints();
-        }
-    }
     return false;
 }
 
