@@ -1,39 +1,34 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -43,12 +38,9 @@
 #include "qdeclarativegeoroute_p.h"
 #include "error_messages.h"
 #include "locationvaluetypehelper_p.h"
-#include <private/qqmlvaluetypewrapper_p.h>
-#include <private/qjsvalue_p.h>
 
 #include <QtCore/QCoreApplication>
 #include <QtQml/QQmlEngine>
-#include <QtQml/QQmlContext>
 #include <QtQml/qqmlinfo.h>
 #include <QtQml/private/qqmlengine_p.h>
 #include <QtLocation/QGeoRoutingManager>
@@ -61,7 +53,7 @@ QT_BEGIN_NAMESPACE
     \instantiates QDeclarativeGeoRouteModel
     \inqmlmodule QtLocation
     \ingroup qml-QtLocation5-routing
-    \since Qt Location 5.0
+    \since Qt Location 5.5
 
     \brief The RouteModel type provides access to routes.
 
@@ -97,7 +89,7 @@ QT_BEGIN_NAMESPACE
     \code
     Plugin {
         id: aPlugin
-        name: "nokia"
+        name: "osm"
     }
 
     RouteQuery {
@@ -158,7 +150,7 @@ int QDeclarativeGeoRouteModel::count() const
 }
 
 /*!
-    \qmlmethod QtLocation::RouteModel::reset()
+    \qmlmethod void QtLocation::RouteModel::reset()
 
     Resets the model. All route data is cleared, any outstanding requests
     are aborted and possible errors are cleared. Model status will be set
@@ -177,13 +169,12 @@ void QDeclarativeGeoRouteModel::reset()
     }
 
     abortRequest();
-    setErrorString(QString());
-    setError(NoError);
+    setError(NoError, QString());
     setStatus(QDeclarativeGeoRouteModel::Null);
 }
 
 /*!
-    \qmlmethod QtLocation::RouteModel::cancel()
+    \qmlmethod void QtLocation::RouteModel::cancel()
 
     Cancels any outstanding requests and clears errors.  Model status will be set to either
     RouteModel.Null or RouteModel.Ready.
@@ -191,8 +182,7 @@ void QDeclarativeGeoRouteModel::reset()
 void QDeclarativeGeoRouteModel::cancel()
 {
     abortRequest();
-    setErrorString(QString());
-    setError(NoError);
+    setError(NoError, QString());
     setStatus(routes_.isEmpty() ? Null : Ready);
 }
 
@@ -210,7 +200,7 @@ void QDeclarativeGeoRouteModel::abortRequest()
 
 
 /*!
-    \qmlmethod QtLocation::RouteModel::get(int)
+    \qmlmethod void QtLocation::RouteModel::get(int)
 
     Returns the Route at given index. Use \l count property to check the
     amount of routes available. The routes are indexed from zero, so the accessible range
@@ -222,8 +212,7 @@ void QDeclarativeGeoRouteModel::abortRequest()
 QDeclarativeGeoRoute *QDeclarativeGeoRouteModel::get(int index)
 {
     if (index < 0 || index >= routes_.count()) {
-        qmlInfo(this) << QStringLiteral("Error, invalid index for get():")
-                      << index;
+        qmlInfo(this) << QStringLiteral("Index '%1' out of range").arg(index);
         return 0;
     }
     return routes_.at(index);
@@ -316,11 +305,31 @@ void QDeclarativeGeoRouteModel::pluginReady()
 {
     QGeoServiceProvider *serviceProvider = plugin_->sharedGeoServiceProvider();
     QGeoRoutingManager *routingManager = serviceProvider->routingManager();
-    if (!routingManager || serviceProvider->error() != QGeoServiceProvider::NoError) {
-        qmlInfo(this) << QStringLiteral("Error: Plugin does not support routing.\nError message:")
-                      << serviceProvider->errorString();
+
+    if (serviceProvider->error() != QGeoServiceProvider::NoError) {
+        QDeclarativeGeoRouteModel::RouteError newError = UnknownError;
+        switch (serviceProvider->error()) {
+        case QGeoServiceProvider::NotSupportedError:
+            newError = EngineNotSetError; break;
+        case QGeoServiceProvider::UnknownParameterError:
+            newError = UnknownParameterError; break;
+        case QGeoServiceProvider::MissingRequiredParameterError:
+            newError = MissingRequiredParameterError; break;
+        case QGeoServiceProvider::ConnectionError:
+            newError = CommunicationError; break;
+        default:
+            break;
+        }
+
+        setError(newError, serviceProvider->errorString());
         return;
     }
+
+    if (!routingManager) {
+        setError(EngineNotSetError, tr("Plugin does not support routing."));
+        return;
+    }
+
     connect(routingManager, SIGNAL(finished(QGeoRouteReply*)),
             this, SLOT(routingFinished(QGeoRouteReply*)));
     connect(routingManager, SIGNAL(error(QGeoRouteReply*,QGeoRouteReply::Error,QString)),
@@ -428,20 +437,16 @@ bool QDeclarativeGeoRouteModel::autoUpdate() const
 */
 void QDeclarativeGeoRouteModel::setMeasurementSystem(QLocale::MeasurementSystem ms)
 {
-    if (!plugin_) {
-        qmlInfo(this) << ROUTE_PLUGIN_NOT_SET;
+    if (!plugin_)
         return;
-    }
 
     QGeoServiceProvider *serviceProvider = plugin_->sharedGeoServiceProvider();
     if (!serviceProvider)
         return;
 
     QGeoRoutingManager *routingManager = serviceProvider->routingManager();
-    if (!routingManager) {
-        qmlInfo(this) << ROUTE_MGR_NOT_SET;
+    if (!routingManager)
         return;
-    }
 
     if (routingManager->measurementSystem() == ms)
         return;
@@ -465,7 +470,6 @@ QLocale::MeasurementSystem QDeclarativeGeoRouteModel::measurementSystem() const
 
     QGeoRoutingManager *routingManager = serviceProvider->routingManager();
     if (!routingManager) {
-        qmlInfo(this) << ROUTE_MGR_NOT_SET;
         if (plugin_->locales().isEmpty())
             return QLocale().measurementSystem();
 
@@ -507,24 +511,10 @@ QDeclarativeGeoRouteModel::Status QDeclarativeGeoRouteModel::status() const
     return status_;
 }
 
-/*!
-    \internal
-*/
-void QDeclarativeGeoRouteModel::setErrorString(const QString &error)
-{
-    if (errorString_ == error)
-        return;
-
-    errorString_ = error;
-
-    if (complete_)
-        emit errorStringChanged();
-}
-
 /*!
     \qmlproperty string QtLocation::RouteModel::errorString
 
-    This read-only property holds the textual presentation of latest routing error.
+    This read-only property holds the textual presentation of the latest routing error.
     If no error has occurred or the model has been reset, an empty string is returned.
 
     An empty string may also be returned if an error occurred which has no associated
@@ -542,12 +532,16 @@ QString QDeclarativeGeoRouteModel::errorString() const
     This read-only property holds the latest error value of the routing request.
 
     \list
-    \li RouteModel.NoError - No error has occurred
-    \li RouteModel.EngineNotSetError - The plugin/service provider used does not support routing
-    \li RouteModel.CommunicationError - An error occurred while communicating with the service provider
-    \li RouteModel.ParseError - The response from the service provider was in an unrecognizable format
-    \li RouteModel.UnsupportedOptionError - The requested operation or one of the options for the operation are not supported by the service provider.
-    \li RouteModel.UnknownError - An error occurred which does not fit into any of the other categories
+    \li RouteModel.NoError - No error has occurred.
+    \li RouteModel.CommunicationError - An error occurred while communicating with the service provider.
+    \li RouteModel.EngineNotSetError - The model's plugin property was not set or there is no routing manager associated with the plugin.
+    \li RouteModel.MissingRequiredParameterError - A required parameter was not specified.
+    \li RouteModel.ParseError - The response from the service provider was in an unrecognizable format.
+    \li RouteModel.UnknownError - An error occurred which does not fit into any of the other categories.
+    \li RouteModel.UnknownParameterError - The plugin did not recognize one of the parameters it was given.
+    \li RouteModel.UnsupportedOptionError - The requested operation is not supported by the routing provider.
+                                            This may happen when the loaded engine does not support a particular
+                                            type of routing request.
     \endlist
 */
 
@@ -556,16 +550,17 @@ QDeclarativeGeoRouteModel::RouteError QDeclarativeGeoRouteModel::error() const
     return error_;
 }
 
-void QDeclarativeGeoRouteModel::setError(RouteError error)
+void QDeclarativeGeoRouteModel::setError(RouteError error, const QString& errorString)
 {
-    if (error_ == error)
+    if (error_ == error && errorString_ == errorString)
         return;
     error_ = error;
+    errorString_ = errorString;
     emit errorChanged();
 }
 
 /*!
-    \qmlmethod QtLocation::RouteModel::update()
+    \qmlmethod void QtLocation::RouteModel::update()
 
     Instructs the RouteModel to update its data. This is most useful
     when \l autoUpdate is disabled, to force a refresh when the query
@@ -577,7 +572,7 @@ void QDeclarativeGeoRouteModel::update()
         return;
 
     if (!plugin_) {
-        qmlInfo(this) << ROUTE_PLUGIN_NOT_SET;
+        setError(EngineNotSetError, tr("Cannot route, plugin not set."));
         return;
     }
 
@@ -587,22 +582,21 @@ void QDeclarativeGeoRouteModel::update()
 
     QGeoRoutingManager *routingManager = serviceProvider->routingManager();
     if (!routingManager) {
-        qmlInfo(this) << ROUTE_MGR_NOT_SET;
+        setError(EngineNotSetError, tr("Cannot route, route manager not set."));
         return;
     }
     if (!routeQuery_) {
-        qmlInfo(this) << QStringLiteral("Cannot route, valid query not set.");
+        setError(ParseError,"Cannot route, valid query not set.");
         return;
     }
     abortRequest(); // Clear previus requests
     QGeoRouteRequest request = routeQuery_->routeRequest();
     if (request.waypoints().count() < 2) {
-        qmlInfo(this) << QStringLiteral("Not enough waypoints for routing.");
+        setError(ParseError,tr("Not enough waypoints for routing."));
         return;
     }
 
-    setErrorString("");   // clear previous error string
-    setError(NoError);
+    setError(NoError, QString());
 
     reply_ = routingManager->calculateRoute(request);
     setStatus(QDeclarativeGeoRouteModel::Loading);
@@ -635,8 +629,7 @@ void QDeclarativeGeoRouteModel::routingFinished(QGeoRouteReply *reply)
     }
     endResetModel();
 
-    setErrorString("");
-    setError(NoError);
+    setError(NoError, QString());
     setStatus(QDeclarativeGeoRouteModel::Ready);
 
     reply->deleteLater();
@@ -657,8 +650,7 @@ void QDeclarativeGeoRouteModel::routingError(QGeoRouteReply *reply,
 {
     if (reply != reply_)
         return;
-    setErrorString(errorString);
-    setError(static_cast<QDeclarativeGeoRouteModel::RouteError>(error));
+    setError(static_cast<QDeclarativeGeoRouteModel::RouteError>(error), errorString);
     setStatus(QDeclarativeGeoRouteModel::Error);
     reply->deleteLater();
     reply_ = 0;
@@ -670,7 +662,7 @@ void QDeclarativeGeoRouteModel::routingError(QGeoRouteReply *reply,
     \instantiates QDeclarativeGeoRouteQuery
     \inqmlmodule QtLocation
     \ingroup qml-QtLocation5-routing
-    \since Qt Location 5.0
+    \since Qt Location 5.5
 
     \brief The RouteQuery type is used to provide query parameters to a
            RouteModel.
@@ -803,21 +795,18 @@ QJSValue QDeclarativeGeoRouteQuery::waypoints()
 {
     QQmlContext *context = QQmlEngine::contextForObject(parent());
     QQmlEngine *engine = context->engine();
-    QV8Engine *v8Engine = QQmlEnginePrivate::getV8Engine(engine);
-    QV4::ExecutionEngine *v4 = QV8Engine::getV4(v8Engine);
+    QV4::ExecutionEngine *v4 = QQmlEnginePrivate::getV4Engine(engine);
 
     QV4::Scope scope(v4);
     QV4::Scoped<QV4::ArrayObject> waypointArray(scope, v4->newArrayObject(request_.waypoints().length()));
     for (int i = 0; i < request_.waypoints().length(); ++i) {
         const QGeoCoordinate &c = request_.waypoints().at(i);
 
-        QQmlValueType *vt = QQmlValueTypeFactory::valueType(qMetaTypeId<QGeoCoordinate>());
-        QV4::ScopedValue cv(scope, QV4::QmlValueTypeWrapper::create(v8Engine, QVariant::fromValue(c), vt));
-
+        QV4::ScopedValue cv(scope, v4->fromVariant(QVariant::fromValue(c)));
         waypointArray->putIndexed(i, cv);
     }
 
-    return new QJSValuePrivate(v4, QV4::ValueRef(waypointArray));
+    return QJSValue(v4, waypointArray.asReturnedValue());
 }
 
 void QDeclarativeGeoRouteQuery::setWaypoints(const QJSValue &value)
@@ -862,21 +851,18 @@ QJSValue QDeclarativeGeoRouteQuery::excludedAreas() const
 {
     QQmlContext *context = QQmlEngine::contextForObject(parent());
     QQmlEngine *engine = context->engine();
-    QV8Engine *v8Engine = QQmlEnginePrivate::getV8Engine(engine);
-    QV4::ExecutionEngine *v4 = QV8Engine::getV4(v8Engine);
+    QV4::ExecutionEngine *v4 = QQmlEnginePrivate::getV4Engine(engine);
 
     QV4::Scope scope(v4);
     QV4::Scoped<QV4::ArrayObject> excludedAreasArray(scope, v4->newArrayObject(request_.excludeAreas().length()));
     for (int i = 0; i < request_.excludeAreas().length(); ++i) {
         const QGeoRectangle &r = request_.excludeAreas().at(i);
 
-        QQmlValueType *vt = QQmlValueTypeFactory::valueType(qMetaTypeId<QGeoRectangle>());
-        QV4::ScopedValue cv(scope, QV4::QmlValueTypeWrapper::create(v8Engine, QVariant::fromValue(r), vt));
-
+        QV4::ScopedValue cv(scope, v4->fromVariant(QVariant::fromValue(r)));
         excludedAreasArray->putIndexed(i, cv);
     }
 
-    return new QJSValuePrivate(v4, QV4::ValueRef(excludedAreasArray));
+    return QJSValue(v4, excludedAreasArray.asReturnedValue());
 }
 
 void QDeclarativeGeoRouteQuery::setExcludedAreas(const QJSValue &value)
@@ -908,7 +894,7 @@ void QDeclarativeGeoRouteQuery::setExcludedAreas(const QJSValue &value)
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::addExcludedArea(georectangle)
+    \qmlmethod void QtLocation::RouteQuery::addExcludedArea(georectangle)
 
     Adds the given area to excluded areas (areas that the route must not cross).
     Same area can only be added once.
@@ -938,7 +924,7 @@ void QDeclarativeGeoRouteQuery::addExcludedArea(const QGeoRectangle &area)
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::removeExcludedArea(georectangle)
+    \qmlmethod void QtLocation::RouteQuery::removeExcludedArea(georectangle)
 
     Removes the given area to excluded areas (areas that the route must not cross).
 
@@ -965,7 +951,7 @@ void QDeclarativeGeoRouteQuery::removeExcludedArea(const QGeoRectangle &area)
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::clearExcludedAreas()
+    \qmlmethod void QtLocation::RouteQuery::clearExcludedAreas()
 
     Clears all excluded areas (areas that the route must not cross).
 
@@ -984,7 +970,7 @@ void QDeclarativeGeoRouteQuery::clearExcludedAreas()
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::addWaypoint(coordinate)
+    \qmlmethod void QtLocation::RouteQuery::addWaypoint(coordinate)
 
     Appends a coordinate to the list of waypoints. Same coordinate
     can be set multiple times.
@@ -1009,7 +995,7 @@ void QDeclarativeGeoRouteQuery::addWaypoint(const QGeoCoordinate &waypoint)
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::removeWaypoint(coordinate)
+    \qmlmethod void QtLocation::RouteQuery::removeWaypoint(coordinate)
 
     Removes the given from the list of waypoints. In case same coordinate
     appears multiple times, the most recently added coordinate instance is
@@ -1036,7 +1022,7 @@ void QDeclarativeGeoRouteQuery::removeWaypoint(const QGeoCoordinate &waypoint)
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::clearWaypoints()
+    \qmlmethod void QtLocation::RouteQuery::clearWaypoints()
 
     Clears all waypoints.
 
@@ -1054,7 +1040,7 @@ void QDeclarativeGeoRouteQuery::clearWaypoints()
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::setFeatureWeight(FeatureType, FeatureWeight)
+    \qmlmethod void QtLocation::RouteQuery::setFeatureWeight(FeatureType, FeatureWeight)
 
     Defines the weight to associate with a feature during the planning of a
     route.
@@ -1096,7 +1082,7 @@ void QDeclarativeGeoRouteQuery::setFeatureWeight(FeatureType featureType, Featur
 }
 
 /*!
-    \qmlmethod QtLocation::RouteQuery::resetFeatureWeights()
+    \qmlmethod void QtLocation::RouteQuery::resetFeatureWeights()
 
     Resets all feature weights to their default state (NeutralFeatureWeight).
 
