@@ -200,9 +200,9 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
     srcPointTypes_.clear();
     srcPointTypes_.reserve(path.size());
 
-    QDoubleVector2D origin, lastPoint, lastAddedPoint;
+    QDoubleVector2D origin, lastAddedPoint;
 
-    const double mapWidthHalf = map.width()/2.0;
+    const double mapWidthHalf = map.viewportWidth()/2.0;
     double unwrapBelowX = 0;
     if (preserveGeometry_)
         unwrapBelowX = map.coordinateToItemPosition(geoLeftBound_, false).x();
@@ -263,8 +263,6 @@ void QGeoMapPolylineGeometry::updateSourcePoints(const QGeoMap &map,
                 lastAddedPoint = point;
             }
         }
-
-        lastPoint = point;
     }
 
     sourceBounds_ = QRectF(QPointF(minX, minY), QPointF(maxX, maxY));
@@ -402,7 +400,7 @@ void QGeoMapPolylineGeometry::updateScreenPoints(const QGeoMap &map,
 
     // Create the viewport rect in the same coordinate system
     // as the actual points
-    QRectF viewport(0, 0, map.width(), map.height());
+    QRectF viewport(0, 0, map.viewportWidth(), map.viewportHeight());
     viewport.adjust(-strokeWidth, -strokeWidth, strokeWidth, strokeWidth);
     viewport.translate(-1 * origin);
 
@@ -965,7 +963,7 @@ MapPolylineNode::MapPolylineNode() :
     geometry_(QSGGeometry::defaultAttributes_Point2D(),0),
     blocked_(true)
 {
-    geometry_.setDrawingMode(GL_TRIANGLE_STRIP);
+    geometry_.setDrawingMode(QSGGeometry::DrawTriangleStrip);
     QSGGeometryNode::setMaterial(&fill_material_);
     QSGGeometryNode::setGeometry(&geometry_);
 }
