@@ -1,39 +1,34 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL3$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPLv3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or later as published by the Free
+** Software Foundation and appearing in the file LICENSE.GPL included in
+** the packaging of this file. Please review the following information to
+** ensure the GNU General Public License version 2.0 requirements will be
+** met: http://www.gnu.org/licenses/gpl-2.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -75,11 +70,11 @@ void QGeoMapItemGeometry::allocateAndFill(QSGGeometry *geom) const
 
     if (isIndexed()) {
         geom->allocate(vx.size(), ix.size());
-        if (geom->indexType() == GL_UNSIGNED_SHORT) {
+        if (geom->indexType() == QSGGeometry::UnsignedShortType) {
             quint16 *its = geom->indexDataAsUShort();
             for (int i = 0; i < ix.size(); ++i)
                 its[i] = ix[i];
-        } else if (geom->indexType() == GL_UNSIGNED_INT) {
+        } else if (geom->indexType() == QSGGeometry::UnsignedIntType) {
             quint32 *its = geom->indexDataAsUInt();
             for (int i = 0; i < ix.size(); ++i)
                 its[i] = ix[i];
@@ -127,15 +122,18 @@ double QGeoMapItemGeometry::geoDistanceToScreenWidth(const QGeoMap &map,
                                                      const QGeoCoordinate &fromCoord,
                                                      const QGeoCoordinate &toCoord)
 {
-    QGeoCoordinate mapMid = map.screenPositionToCoordinate(QDoubleVector2D(map.width()/2.0, 0));
+    // Do not wrap around half the globe
+    Q_ASSERT(!qFuzzyCompare(fromCoord.longitude(), toCoord.longitude()));
+
+    QGeoCoordinate mapMid = map.itemPositionToCoordinate(QDoubleVector2D(map.viewportWidth()/2.0, 0));
     double halfGeoDist = toCoord.longitude() - fromCoord.longitude();
     if (toCoord.longitude() < fromCoord.longitude())
         halfGeoDist += 360;
     halfGeoDist /= 2.0;
     QGeoCoordinate geoDelta =  QGeoCoordinate(0,
                     QLocationUtils::wrapLong(mapMid.longitude() + halfGeoDist));
-    QDoubleVector2D halfScreenDist = map.coordinateToScreenPosition(geoDelta, false)
-                                - QDoubleVector2D(map.width()/2.0, 0);
+    QDoubleVector2D halfScreenDist = map.coordinateToItemPosition(geoDelta, false)
+                                - QDoubleVector2D(map.viewportWidth()/2.0, 0);
     return halfScreenDist.x() * 2.0;
 }
 
