@@ -40,7 +40,7 @@
 #include "qgeotilespec_p.h"
 #include <QtPositioning/private/qdoublevector3d_p.h>
 #include <QtCore/private/qobject_p.h>
-#include <QtQuick/QSGSimpleTextureNode>
+#include <QtQuick/QSGImageNode>
 #include <QtQuick/QQuickWindow>
 #include <cmath>
 
@@ -101,6 +101,8 @@ public:
     bool m_verticalLock;
     bool m_linearScaling;
 
+    bool m_dropTextures;
+
     void addTile(const QGeoTileSpec &spec, QSharedPointer<QGeoTileTexture> texture);
 
     QDoubleVector2D itemPositionToMercator(const QDoubleVector2D &pos) const;
@@ -108,7 +110,7 @@ public:
 
     void setVisibleTiles(const QSet<QGeoTileSpec> &tiles);
     void removeTiles(const QSet<QGeoTileSpec> &oldTiles);
-    bool buildGeometry(const QGeoTileSpec &spec, QSGGeometry::TexturedPoint2D *vertices);
+    bool buildGeometry(const QGeoTileSpec &spec, QSGImageNode *imageNode);
     void setTileBounds(const QSet<QGeoTileSpec> &tiles);
     void setupCamera();
 };
@@ -200,6 +202,7 @@ void QGeoTiledMapScene::clearTexturedTiles()
 {
     Q_D(QGeoTiledMapScene);
     d->m_textures.clear();
+    d->m_dropTextures = true;
 }
 
 QGeoTiledMapScenePrivate::QGeoTiledMapScenePrivate()
@@ -223,7 +226,8 @@ QGeoTiledMapScenePrivate::QGeoTiledMapScenePrivate()
       m_screenHeight(0.0),
       m_useVerticalLock(false),
       m_verticalLock(false),
-      m_linearScaling(false)
+      m_linearScaling(false),
+      m_dropTextures(false)
 {
 }
 
@@ -292,7 +296,7 @@ QDoubleVector2D QGeoTiledMapScenePrivate::mercatorToItemPosition(const QDoubleVe
     return QDoubleVector2D(x + m_screenOffsetX, y + m_screenOffsetY);
 }
 
-bool QGeoTiledMapScenePrivate::buildGeometry(const QGeoTileSpec &spec, QSGGeometry::TexturedPoint2D *vertices)
+bool QGeoTiledMapScenePrivate::buildGeometry(const QGeoTileSpec &spec, QSGImageNode *imageNode)
 {
     int x = spec.x();
 
@@ -320,11 +324,9 @@ bool QGeoTiledMapScenePrivate::buildGeometry(const QGeoTileSpec &spec, QSGGeomet
     y1 *= edge;
     y2 *= edge;
 
-    //Texture coordinate order for veritcal flip of texture
-    vertices[0].set(x1, y1, 0, 0);
-    vertices[1].set(x1, y2, 0, 1);
-    vertices[2].set(x2, y1, 1, 0);
-    vertices[3].set(x2, y2, 1, 1);
+    imageNode->setRect(QRectF(QPointF(x1, y2), QPointF(x2, y1)));
+    imageNode->setTextureCoordinatesTransform(QSGImageNode::MirrorVertically);
+    imageNode->setSourceRect(QRectF(QPointF(0,0), imageNode->texture()->textureSize()));
 
     return true;
 }
@@ -559,12 +561,12 @@ void QGeoTiledMapScenePrivate::setupCamera()
 class QGeoTiledMapTileContainerNode : public QSGTransformNode
 {
 public:
-    void addChild(const QGeoTileSpec &spec, QSGSimpleTextureNode *node)
+    void addChild(const QGeoTileSpec &spec, QSGImageNode *node)
     {
         tiles.insert(spec, node);
         appendChildNode(node);
     }
-    QHash<QGeoTileSpec, QSGSimpleTextureNode *> tiles;
+    QHash<QGeoTileSpec, QSGImageNode *> tiles;
 };
 
 class QGeoTiledMapRootNode : public QSGClipNode
@@ -601,7 +603,10 @@ public:
         }
     }
 
-    void updateTiles(QGeoTiledMapTileContainerNode *root, QGeoTiledMapScenePrivate *d, double camAdjust);
+    void updateTiles(QGeoTiledMapTileContainerNode *root,
+                     QGeoTiledMapScenePrivate *d,
+                     double camAdjust,
+                     QQuickWindow *window);
 
     bool isTextureLinear;
 
@@ -617,11 +622,9 @@ public:
     QHash<QGeoTileSpec, QSGTexture *> textures;
 };
 
-static bool qgeotiledmapscene_isTileInViewport(const QSGGeometry::TexturedPoint2D *tp, const QMatrix4x4 &matrix) {
-    QPolygonF polygon; polygon.reserve(4);
-    for (int i=0; i<4; ++i)
-        polygon << matrix * QPointF(tp[i].x, tp[i].y);
-    return QRectF(-1, -1, 2, 2).intersects(polygon.boundingRect());
+static bool qgeotiledmapscene_isTileInViewport(const QRectF &tileRect, const QMatrix4x4 &matrix) {
+    const QRectF boundingRect = QRectF(matrix * tileRect.topLeft(), matrix * tileRect.bottomRight());
+    return QRectF(-1, -1, 2, 2).intersects(boundingRect);
 }
 
 static QVector3D toVector3D(const QDoubleVector3D& in)
@@ -630,8 +633,9 @@ static QVector3D toVector3D(const QDoubleVector3D& in)
 }
 
 void QGeoTiledMapRootNode::updateTiles(QGeoTiledMapTileContainerNode *root,
-                                  QGeoTiledMapScenePrivate *d,
-                                  double camAdjust)
+                                       QGeoTiledMapScenePrivate *d,
+                                       double camAdjust,
+                                       QQuickWindow *window)
 {
     // Set up the matrix...
     QDoubleVector3D eye = d->m_cameraEye;
@@ -649,30 +653,24 @@ void QGeoTiledMapRootNode::updateTiles(QGeoTiledMapTileContainerNode *root,
     foreach (const QGeoTileSpec &s, toRemove)
         delete root->tiles.take(s);
 
-    for (QHash<QGeoTileSpec, QSGSimpleTextureNode *>::iterator it = root->tiles.begin();
+    for (QHash<QGeoTileSpec, QSGImageNode *>::iterator it = root->tiles.begin();
          it != root->tiles.end(); ) {
-        QSGGeometry visualGeometry(QSGGeometry::defaultAttributes_TexturedPoint2D(), 4);
-        QSGGeometry::TexturedPoint2D *v = visualGeometry.vertexDataAsTexturedPoint2D();
-        bool ok = d->buildGeometry(it.key(), v) && qgeotiledmapscene_isTileInViewport(v, root->matrix());
-        QSGSimpleTextureNode *node = it.value();
-        QSGNode::DirtyState dirtyBits = 0;
+        QSGImageNode *node = it.value();
+        bool ok = d->buildGeometry(it.key(), node) && qgeotiledmapscene_isTileInViewport(node->rect(), root->matrix());
 
-        // Check and handle changes to vertex data.
-        if (ok && memcmp(node->geometry()->vertexData(), v, 4 * sizeof(QSGGeometry::TexturedPoint2D)) != 0) {
-            if (v[0].x == v[3].x || v[0].y == v[3].y) { // top-left == bottom-right => invalid => remove
-                ok = false;
-            } else {
-                memcpy(node->geometry()->vertexData(), v, 4 * sizeof(QSGGeometry::TexturedPoint2D));
-                dirtyBits |= QSGNode::DirtyGeometry;
-            }
-        }
+        QSGNode::DirtyState dirtyBits = 0;
 
         if (!ok) {
             it = root->tiles.erase(it);
             delete node;
         } else {
             if (isTextureLinear != d->m_linearScaling) {
-                node->setFiltering(d->m_linearScaling ? QSGTexture::Linear : QSGTexture::Nearest);
+                if (node->texture()->textureSize().width() > d->m_tileSize) {
+                    node->setFiltering(QSGTexture::Linear); // With mipmapping QSGTexture::Nearest generates artifacts
+                    node->setMipmapFiltering(QSGTexture::Linear);
+                } else {
+                    node->setFiltering(d->m_linearScaling ? QSGTexture::Linear : QSGTexture::Nearest);
+                }
                 dirtyBits |= QSGNode::DirtyMaterial;
             }
             if (dirtyBits != 0)
@@ -685,15 +683,16 @@ void QGeoTiledMapRootNode::updateTiles(QGeoTiledMapTileContainerNode *root,
         QGeoTileTexture *tileTexture = d->m_textures.value(s).data();
         if (!tileTexture || tileTexture->image.isNull())
             continue;
-        QSGSimpleTextureNode *tileNode = new QSGSimpleTextureNode();
+        QSGImageNode *tileNode = window->createImageNode();
         // note: setTexture will update coordinates so do it here, before we buildGeometry
         tileNode->setTexture(textures.value(s));
-        Q_ASSERT(tileNode->geometry());
-        Q_ASSERT(tileNode->geometry()->attributes() == QSGGeometry::defaultAttributes_TexturedPoint2D().attributes);
-        Q_ASSERT(tileNode->geometry()->vertexCount() == 4);
-        if (d->buildGeometry(s, tileNode->geometry()->vertexDataAsTexturedPoint2D())
-                && qgeotiledmapscene_isTileInViewport(tileNode->geometry()->vertexDataAsTexturedPoint2D(), root->matrix())) {
-            tileNode->setFiltering(d->m_linearScaling ? QSGTexture::Linear : QSGTexture::Nearest);
+        if (d->buildGeometry(s, tileNode) && qgeotiledmapscene_isTileInViewport(tileNode->rect(), root->matrix())) {
+            if (tileNode->texture()->textureSize().width() > d->m_tileSize) {
+                tileNode->setFiltering(QSGTexture::Linear); // with mipmapping QSGTexture::Nearest generates artifacts
+                tileNode->setMipmapFiltering(QSGTexture::Linear);
+            } else {
+                tileNode->setFiltering(d->m_linearScaling ? QSGTexture::Linear : QSGTexture::Nearest);
+            }
             root->addChild(s, tileNode);
         } else {
             delete tileNode;
@@ -723,6 +722,18 @@ QSGNode *QGeoTiledMapScene::updateSceneGraph(QSGNode *oldNode, QQuickWindow *win
     itemSpaceMatrix.scale(1, -1);
     mapRoot->root->setMatrix(itemSpaceMatrix);
 
+    if (d->m_dropTextures) {
+        foreach (const QGeoTileSpec &s, mapRoot->tiles->tiles.keys())
+            delete mapRoot->tiles->tiles.take(s);
+        foreach (const QGeoTileSpec &s, mapRoot->wrapLeft->tiles.keys())
+            delete mapRoot->wrapLeft->tiles.take(s);
+        foreach (const QGeoTileSpec &s, mapRoot->wrapRight->tiles.keys())
+            delete mapRoot->wrapRight->tiles.take(s);
+        foreach (const QGeoTileSpec &spec, mapRoot->textures.keys())
+            mapRoot->textures.take(spec)->deleteLater();
+        d->m_dropTextures = false;
+    }
+
     QSet<QGeoTileSpec> textures = QSet<QGeoTileSpec>::fromList(mapRoot->textures.keys());
     QSet<QGeoTileSpec> toRemove = textures - d->m_visibleTiles;
     QSet<QGeoTileSpec> toAdd = d->m_visibleTiles - textures;
@@ -737,9 +748,9 @@ QSGNode *QGeoTiledMapScene::updateSceneGraph(QSGNode *oldNode, QQuickWindow *win
     }
 
     double sideLength = d->m_scaleFactor * d->m_tileSize * d->m_sideLength;
-    mapRoot->updateTiles(mapRoot->tiles, d, 0);
-    mapRoot->updateTiles(mapRoot->wrapLeft, d, +sideLength);
-    mapRoot->updateTiles(mapRoot->wrapRight, d, -sideLength);
+    mapRoot->updateTiles(mapRoot->tiles, d, 0, window);
+    mapRoot->updateTiles(mapRoot->wrapLeft, d, +sideLength, window);
+    mapRoot->updateTiles(mapRoot->wrapRight, d, -sideLength, window);
 
     mapRoot->isTextureLinear = d->m_linearScaling;
 
